/**
 * Copyright 2004-2006 Fabrizio Giustina.
 *
 * Licensed under the Artistic License; you may not use this file
 * except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://maven-taglib.sourceforge.net/license.html
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sf.maventaglib;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;

import net.sf.maventaglib.util.XmlHelper;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.util.FileUtils;
import org.codehaus.plexus.util.IOUtil;
import org.jdom.input.DOMBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.sun.tlddoc.tagfileparser.Attribute;
import com.sun.tlddoc.tagfileparser.Directive;
import com.sun.tlddoc.tagfileparser.ParseException;
import com.sun.tlddoc.tagfileparser.TagFile;

/**
 * Generates tld files from directories of jsp 2.0 tag files.
 * @goal tldgenerate
 * @phase generate-resources
 * @author Fabrizio Giustina
 * @version $Id: TldGenerateMojo.java 189 2006-08-20 11:59:53Z fgiust $
 */
public class TldGenerateMojo
    extends AbstractMojo
{
    /**
     * Directory containing tag files. Subdirectories are also processed.
     * @parameter expression="src/main/resources/META-INF/tags/"
     */
    private File tagDir;

    /**
     * Output dir for tld files.
     * @parameter expression="${project.build.outputDirectory}/META-INF"
     * @required
     */
    private File outputDir;

    /**
     * Version added to tld files, defaults to project version.
     * @parameter expression="${project.version}"
     * @required
     */
    private String version;

    /**
     * @see org.apache.maven.plugin.Mojo#execute()
     */
    public void execute()
        throws MojoExecutionException, MojoFailureException
    {
        getLog().debug( MessageFormat.format( Messages.getString( "Taglib.generating.tld" ), //$NON-NLS-1$
                                              new Object[] { tagDir.getAbsolutePath() } ) );

        if ( !tagDir.isDirectory() )
        {
            throw new MojoExecutionException( MessageFormat
                .format( Messages.getString( "Taglib.notadir" ), new Object[] { tagDir //$NON-NLS-1$
                    .getAbsolutePath() } ) );
        }

        try
        {
            // handle tag files. Add any directory containing .tag or .tagx files
            List tags = FileUtils.getFiles( tagDir, "**/*.tag", null ); //$NON-NLS-1$
            tags.addAll( FileUtils.getFiles( tagDir, "**/*.tagx", null ) ); //$NON-NLS-1$

            if ( !tags.isEmpty() )
            {
                Set directories = new HashSet();
                for ( Iterator it = tags.iterator(); it.hasNext(); )
                {
                    directories.add( ( (File) it.next() ).getParentFile() );
                }

                XMLOutputter outputter = new XMLOutputter( Format.getPrettyFormat() );

                for ( Iterator it = directories.iterator(); it.hasNext(); )
                {
                    File tagDir = (File) it.next();

                    try
                    {
                        Document doc = getTLDDocument( tagDir, XmlHelper.getDocumentBuilder() );

                        String tldName = tagDir.getName() + ".tld";
                        File outputFile = new File( outputDir, tldName );
                        outputDir.mkdirs();

                        getLog().info( MessageFormat.format( Messages.getString( "Taglib.generating.file" ), //$NON-NLS-1$
                                                             new Object[] { tldName, tagDir.getAbsolutePath() } ) );

                        FileOutputStream fos = new FileOutputStream( outputFile );
                        try
                        {
                            outputter.output( new DOMBuilder().build( doc ), fos );
                        }
                        finally
                        {
                            IOUtil.close( fos );
                        }
                    }
                    catch ( Exception e )
                    {
                        throw new MojoExecutionException( e.getMessage(), e );
                    }
                }
            }
            else
            {
                getLog().warn( MessageFormat.format( Messages.getString( "Taglib.generating.notfound" ), //$NON-NLS-1$
                                                     new Object[] { tagDir.getAbsolutePath() } ) );
            }

        }
        catch ( IOException e )
        {
            throw new MojoExecutionException( e.getMessage(), e );
        }

    }

    protected Document getTLDDocument( File dir, DocumentBuilder documentBuilder )

    {
        Document result = documentBuilder.newDocument();

        String path = dir.getAbsolutePath().replace( File.separatorChar, '/' );
        int index = path.indexOf( "/META-INF/" );
        if ( index != -1 )
        {
            path = path.substring( index );
        }
        else
        {
            path = "unknown";
        }
        if ( !path.endsWith( "/" ) )
        {
            path += "/";
        }

        String description = "Tag library for tag file directory " + path;

        String shortName = path;
        if ( shortName.startsWith( "/META-INF/tags" ) )
        {
            shortName = shortName.substring( "/META-INF/tags".length() );
        }

        shortName = shortName.replace( File.separatorChar, '/' );
        shortName = StringUtils.strip( shortName, "/" );

        shortName = shortName.replace( '/', '-' );

        String uri = shortName;

        Element taglibElement = createRootTaglibNode( result, path, description, shortName, uri );

        File[] files = dir.listFiles();
        for ( int j = 0; ( files != null ) && ( j < files.length ); j++ )
        {
            File tag = files[j];

            if ( !tag.isDirectory()
                && ( tag.getName().toLowerCase().endsWith( ".tag" ) || tag.getName().toLowerCase().endsWith( ".tagx" ) ) )
            {
                String tagName = tag.getName().substring( 0, tag.getName().lastIndexOf( '.' ) );
                String tagPath = path + tag.getName();

                Element tagFileElement = result.createElement( "tag-file" );
                Element nameElement = result.createElement( "name" );
                nameElement.appendChild( result.createTextNode( tagName ) );
                tagFileElement.appendChild( nameElement );
                Element pathElement = result.createElement( "path" );
                pathElement.appendChild( result.createTextNode( tagPath ) );
                tagFileElement.appendChild( pathElement );
                taglibElement.appendChild( tagFileElement );
                tag.getName();

                //  tag-file Subelements
                //
                //  description (optional) A description of the tag.
                //  display-name (optional) Name intended to be displayed by tools.
                //  icon (optional) Icon that can be used by tools.
                //  name The unique tag name.
                //  path Where to find the tag file implementing this tag, relative to the root of the web application or the root
                //     of the JAR file for a tag library packaged in a JAR. This must begin with /WEB-INF/tags/ if the tag file resides
                //     in the WAR, or /META-INF/tags/ if the tag file resides in a JAR.
                //  example (optional) Informal description of an example use of the tag.
                //  tag-extension (optional) Extensions that provide extra information about the tag for tools.

                InputStream is = null;
                try
                {
                    is = new FileInputStream( tag );
                }
                catch ( FileNotFoundException e )
                {
                    // @todo Auto-generated catch block
                    e.printStackTrace();
                }

                try
                {
                    TagFile tagFile = TagFile.parse( is );

                    for ( Iterator iter = tagFile.getDirectives().iterator(); iter.hasNext(); )
                    {
                        Directive directive = (Directive) iter.next();
                        if ( "tag".equals( directive.getDirectiveName() ) )
                        {
                            for ( Iterator iterator = directive.getAttributes(); iterator.hasNext(); )
                            {
                                Attribute attribute = (Attribute) iterator.next();
                                if ( "description".equals( attribute.getName() )
                                    || "display-name".equals( attribute.getName() )
                                    || "example".equals( attribute.getName() ) )
                                {
                                    Element element = result.createElement( attribute.getName() );
                                    element.appendChild( result.createTextNode( attribute.getValue() ) );
                                    tagFileElement.appendChild( element );
                                }

                            }

                        }

                    }

                }
                catch ( ParseException e )
                {
                    // @todo Auto-generated catch block
                    e.printStackTrace();
                }
                finally
                {
                    IOUtil.close( is );
                }

            }
        }

        return result;
    }

    protected Element createRootTaglibNode( Document result, String path, String description, String shortName,
                                            String uri )
    {
        Element taglibElement = result.createElementNS( "http://java.sun.com/xml/ns/javaee", "taglib" );
        taglibElement.setAttributeNS( "http://www.w3.org/2000/xmlns/", "xmlns", "http://java.sun.com/xml/ns/javaee" );
        taglibElement.setAttributeNS( "http://www.w3.org/2000/xmlns/", "xmlns:xsi",
                                      "http://www.w3.org/2001/XMLSchema-instance" );
        taglibElement
            .setAttributeNS( "http://www.w3.org/2001/XMLSchema-instance", "xsi:schemaLocation",
                             "http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-jsptaglibrary_2_1.xsd" );
        taglibElement.setAttribute( "version", "2.1" );
        result.appendChild( taglibElement );

        Element descriptionElement = result.createElement( "description" );
        descriptionElement.appendChild( result.createTextNode( description ) );
        taglibElement.appendChild( descriptionElement );

        Element tlibVersionElement = result.createElement( "tlib-version" );
        tlibVersionElement.appendChild( result.createTextNode( version ) );
        taglibElement.appendChild( tlibVersionElement );

        Element shortNameElement = result.createElement( "short-name" );
        shortNameElement.appendChild( result.createTextNode( shortName ) );
        taglibElement.appendChild( shortNameElement );

        Element uriElement = result.createElement( "uri" );
        uriElement.appendChild( result.createTextNode( uri ) );
        taglibElement.appendChild( uriElement );

        return taglibElement;
    }
}
